package com.ekingwin.bas.cloud.document.service.impl;

import java.util.Date;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ekingwin.bas.cloud.document.DocManagerException;
import com.ekingwin.bas.cloud.document.dao.entity.DocFile;
import com.ekingwin.bas.cloud.document.dao.entity.DocumentInfo;
import com.ekingwin.bas.cloud.document.dao.entity.DocumentMeta;
import com.ekingwin.bas.cloud.document.dto.UploadUserDto;
import com.ekingwin.bas.cloud.document.model.DocModel;
import com.ekingwin.bas.cloud.document.model.FileModel;
import com.ekingwin.bas.cloud.document.service.IDocFileService;
import com.ekingwin.bas.cloud.document.service.IDocService;
import com.ekingwin.bas.cloud.document.service.IDocumentMetaService;
import com.ekingwin.bas.cloud.document.service.IUserService;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.infra.utils.common.DateUtil;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;


@Primary
@Service
public class DocServiceImpl extends BaseService implements IDocService {

	@Autowired
	private IDocFileService docFileService;

	@Autowired
	@Qualifier(value = "userServiceImpl")
	private IUserService userManager;
	
	@Autowired
	private IDocumentMetaService docMetaService;

	
	@Override
	@Transactional
	public DocModel checkIn(UploadUserDto user, DocModel docModel) throws DocManagerException {
		
		String today = DateUtil.formatDate(new Date().getTime(), "yyyy-MM-dd HH:mm:ss");
		
		docModel.setInDate(today);
		if(StringUtils.isEmpty(docModel.getCreateDate())){
			docModel.setCreateDate(today);
		}
		
		DocumentInfo doc = this.buildDocument(user, docModel);
		
		// 保存文件本体
		DocFile df = docFileService.storeFile(doc);
		
		df.setOriginalName(doc.getFileOriginalName());
		// 保存文件信息到数据库
		docFileService.save(df);
		// 设置保存后的文件ID
		doc.setFileId(df.getId());
		// 初始化文档元数据对象
		DocumentMeta docMeta = buildDocMeta(doc);
		// 保存文档元数据
		docMetaService.save(docMeta);
		
		// 设置文档元数据保存后的id
		doc.setdId(docMeta.getId());
				
		DocFile docfile = docFileService.get(docMeta.getFileId() + "");
		docModel = this.buildDocModel(doc);
		return docModel;
	}
	
	private DocumentMeta buildDocMeta(DocumentInfo doc) {
		DocumentMeta docMeta = new DocumentMeta();
		docMeta.setAttachments(doc.getAttachments());
		docMeta.setAuthor(doc.getAuthor());
		docMeta.setComments(doc.getComments());
		docMeta.setFolderId(doc.getFolderId());
		docMeta.setImage(doc.getReviewImg());
		docMeta.setTitle(doc.getTitle());
		docMeta.setTags(doc.getTags());
		docMeta.setFileId(doc.getFileId());
		docMeta.setStatus("3"); //3代表初始状态
		docMeta.setHidden("FALSE");
		docMeta.setReadonly("FALSE");
		docMeta.setCreateon(doc.getCreateDate());
		docMeta.setCreateby(doc.getAuthor());
		docMeta.setDocno(doc.getDocno());
		docMeta.setWatermark(doc.getWatermark());
		if (doc.getFileId() != null) {
			docMeta.setFileId(doc.getFileId());
		}
		if (doc.getdId() != null) {
			docMeta.setId(doc.getdId());
		}
		return docMeta;
	}
	
	private DocModel buildDocModel(DocumentInfo docInfo) {
		DocModel model = new DocModel();
		
		model.setDocTitle(docInfo.getTitle());
		model.setComments(docInfo.getComments());
		model.setDocId(docInfo.getdId() + "");
		model.setDocName(docInfo.getDocName());
		model.setDocType(docInfo.getDocType());
		model.setDocCreator(docInfo.getCheckInUser());
		model.setDocAuthor(docInfo.getAuthor());
		model.setDocAuthorDis(docInfo.getAuthorDisplay());
		model.setDownloadUrl(docInfo.getDownloadUrl());
		model.setWebUrl(docInfo.getViewUrl());
		model.setRevlabel(docInfo.getRevisionId() + "");
		model.setOriginalName(docInfo.getFileOriginalName());
		model.setDocno(docInfo.getDocno());
		model.setWatermark(docInfo.getWatermark());
		model.setRevId(docInfo.getRevId());
		if(docInfo.getCreateDate() != null){
			String createDate = DateUtil.formatDate(docInfo.getCreateDate().getTime(), "yyyy-MM-dd HH:mm:ss");
			model.setCreateDate(createDate);
		}
		model.setFolderIds(docInfo.getFolderids());
		model.setFolderName(docInfo.getFoldernames());
		model.setFileId(docInfo.getFileId() + "");
		model.setDownSum(docInfo.getDownSum());
		model.setReadSum(docInfo.getReadSum());
		model.setTags(docInfo.getTags());
		model.setProvideruser(docInfo.getProvideruser());
		model.setDiscuss(docInfo.getDiscuss());
		model.setCollect(docInfo.getCollect());
		model.setScore(docInfo.getScore());
		
		if(docInfo.getReleaseDate() != null){
			String releaseDate = DateUtil.formatDate(docInfo.getReleaseDate().getTime(), "yyyy-MM-dd HH:mm:ss");
			try {
				model.setInDate(releaseDate);
			} catch (DocManagerException e) {
				e.printStackTrace();
			}
			model.getAttrs().put("xpublishTime", releaseDate);
		}
		
		model.getAttrs().put("xpublishPeople", docInfo.getCheckInUser());
		model.getAttrs().put("xdocImage", docInfo.getReviewImg());
		model.getAttrs().put("xattachmentID", docInfo.getAttachments());
		
		if(docInfo.getExtAttr() != null){
			model.getAttrs().putAll(docInfo.getExtAttr());
		}
		return model;
	}

	private DocumentInfo buildDocument(UploadUserDto user, DocModel docModel) {
		DocumentInfo doc = new DocumentInfo();
		doc.setTitle(docModel.getDocTitle());
		doc.setComments(docModel.getComments());
		doc.setDocno(docModel.getDocno());
		doc.setDocType(docModel.getDocType());
		doc.setWatermark(docModel.getWatermark());
		if(!StringUtils.isEmpty(docModel.getFolderId())){
			doc.setFolderId(docModel.getFolderId());
		}
		doc.setReviewImg(docModel.getAttrs().get("xdocImage"));
		if(StringUtils.isEmpty(docModel.getDocAuthor())){
			doc.setAuthor(user.getEmpid());
		}else{
			doc.setAuthor(docModel.getDocAuthor());
		}
		
		doc.setAttachments(docModel.getAttrs().get("xattachmentID"));
		
		doc.setTags(docModel.getTags());
		
		
		
		doc.setDocName(docModel.getDocName());
		if(docModel.getDocId() != null){
			doc.setdId(docModel.getDocId());
		}
		
		doc.setCheckInUser(user.getEmpid());
		
		doc.setFileLocation(docModel.getPrimaryFile());
		
		doc.setFileOriginalName(docModel.getOriginalName());
		
		if(StringUtils.isEmpty(docModel.getReleaseDate())){
			doc.setReleaseDate(DateUtil.parseDate(docModel.getCreateDate(), "yyyy-MM-dd HH:mm:ss"));
		}else{
			doc.setReleaseDate(DateUtil.parseDate(docModel.getReleaseDate(), "yyyy-MM-dd HH:mm:ss"));
		}
		
		doc.setCreateDate(DateUtil.parseDate(docModel.getCreateDate(), "yyyy-MM-dd HH:mm:ss"));
		doc.setExpiredDate(DateUtil.parseDate(docModel.getExpiredDate(), "yyyy-MM-dd HH:mm:ss"));
		
		
		
		if(!StringUtils.isEmpty(docModel.getFileId())){
			doc.setFileId(docModel.getFileId());
		}
		
		doc.setExtAttr(docModel.getAttrs());
		
		return doc;
	}
	

	@Override
	public DocModel getDocModelBydID(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public DocModel getDocModelBydName(UploadUserDto user, String dName) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public DocModel getDocModelBasicBydID(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public FileModel getPrimaryFileBydID(UploadUserDto user, String dID, String fileName, FileModel fileModel) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public FileModel getWebFileBydID(UploadUserDto user, String dID, String fileName, FileModel fileModel) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public Map<String, Object> searchByQueryText(String userId, String queryText) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public Map<String, Object> searchByQueryText(String userId, String queryText, String orderBy, String sort,
			Integer rowCount, Integer startRow) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public PageCriteria<DocModel> search(QueryDTO query) throws DocManagerException {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public void checkOutBydID(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void unCheckOutBydID(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public DocModel updataDocModel(UploadUserDto user, DocModel docModel) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public DocModel updataNewRevDocModel(UploadUserDto user, DocModel docModel) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public void updataDocMeda(String userId, Map<String, String> attrs) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void deleteDocModelAllRevision(UploadUserDto user, String dID, String docName) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void deleteNoReconver(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void deleteDocModelBydId(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void batchDelete(UploadUserDto user, String[] docids) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void physicsDelete(String userId, String dID) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void recover(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void subscription(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void unSubscription(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void collection(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void unCollection(UploadUserDto user, String dID) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public DocModel getFullDocModelBydID(UploadUserDto user, String docId, String revId) throws BasException {
		// TODO Auto-generated method stub
		return null;
	}

	
}
